package model

import (
	"encoding/json"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"time"
)

type TowerAlarmConf struct {
	ID int64 `gorm:"column:ID;PRIMARY_KEY" `
	SN string `gorm:"column:SN"`
	ProjectID int64 `gorm:"column:ProjectID" `
	HorizontalDistanceAlarm float64 `gorm:"column:HorizontalDistanceAlarm" `
	VerticalDistanceAlarm float64 `gorm:"column:VerticalDistanceAlarm" `
	WeightAlarm int64 `gorm:"column:WeightAlarm" `
	MomentAlarm float64 `gorm:"column:MomentAlarm" `
	WindSpeedAlarm float64 `gorm:"column:WindSpeedAlarm" `
	TiltAlarm float64 `gorm:"column:TiltAlarm" json:""`
	HorizontalDistanceWarning float64 `gorm:"column:HorizontalDistanceWarning" `
	VerticalDistanceWarning float64 `gorm:"column:VerticalDistanceWarning"`
	WeightWarning int64 `gorm:"column:WeightWarning" `
	MomentWarning float64 `gorm:"column:MomentWarning" `
	WindSpeedWarning float64 `gorm:"column:WindSpeedWarning" `
	TiltWarning float64 `gorm:"column:TiltWarning" `
	TowerNo int `gorm:"column:TowerNo"`
	CreatedAt time.Time `gorm:"column:CreatedAt"`
	UpdatedAt time.Time `gorm:"column:UpdatedAt"`
}

func (TowerAlarmConf) TableName() string {
	return "TowerAlarmConf"
}

func (p *TowerAlarmConf) Insert(db *gorm.DB) error {
	err := db.Create(p).Error
	if err != nil {
		fields, _ := json.Marshal(p)
		logger.Error("mysql",
			zap.String("sql", "insert into TowerAlarmConf"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return err
	}
	return err
}

func (p *TowerAlarmConf) Find(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	err = db.Table(p.TableName()).Where(cond, val...).First(p).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "select from TowerAlarmConf"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return err
	}
	return err
}

func (p *TowerAlarmConf) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}

	err = db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		updateValues, _ := json.Marshal(values)
		logger.Error("mysql",
			zap.String("sql", "update TowerAlarmConf"),
			zap.String("fields", string(fields)),
			zap.String("values", string(updateValues)),
			zap.String("error", err.Error()))
		return err
	}
	return err
}

func (p *TowerAlarmConf) Count(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) (int64, error) {

	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return 0, err
	}
	ret := int64(0)
	err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
	if err != nil {
		fields, _ := json.Marshal(val)
		logger.Error("mysql",
			zap.String("sql", "select count(1) TowerAlarmConf"),
			zap.String("fields", cond+","+string(fields)),
			zap.String("error", err.Error()))
		return ret, err
	}

	return ret, err
}

