package model

import (
	"encoding/json"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"time"
)

type TowerPositionConf struct {
	ID int64 `gorm:"column:ID;PRIMARY_KEY"`
	SN string `gorm:"column:SN"`
	ProjectID int64 `gorm:"column:ProjectID"`
	RangePositionStart float64 `gorm:"column:RangePositionStart"`
	RangePositionEnd float64 `gorm:"column:RangePositionEnd" `
	HighPositionStart float64 `gorm:"column:HighPositionStart" `
	HighPositionEnd float64 `gorm:"column:HighPositionEnd" json:""`
	BackturnPositionStart float64 `gorm:"column:BackturnPositionStart"`
	BackturnPositionEnd float64 `gorm:"column:BackturnPositionEnd"`
	TowerNo int `gorm:"column:TowerNo"`
	CreatedAt time.Time `gorm:"column:CreatedAt"`
	UpdatedAt time.Time `gorm:"column:UpdatedAt"`
}

func (TowerPositionConf) TableName() string {
	return "TowerPositionConf"
}

func (p *TowerPositionConf) Insert(db *gorm.DB) error {
	err := db.Create(p).Error
	if err != nil {
		fields, _ := json.Marshal(p)
		logger.Error("mysql",
			zap.String("sql", "insert into TowerPositionConf"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return err
	}
	return err
}

func (p *TowerPositionConf) Find(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	err = db.Table(p.TableName()).Where(cond, val...).First(p).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "select from TowerPositionConf"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return err
	}
	return err
}

func (p *TowerPositionConf) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}

	err = db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		updateValues, _ := json.Marshal(values)
		logger.Error("mysql",
			zap.String("sql", "update TowerPositionConf"),
			zap.String("fields", string(fields)),
			zap.String("values", string(updateValues)),
			zap.String("error", err.Error()))
		return err
	}
	return err
}

func (p *TowerPositionConf) Count(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) (int64, error) {

	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return 0, err
	}
	ret := int64(0)
	err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
	if err != nil {
		fields, _ := json.Marshal(val)
		logger.Error("mysql",
			zap.String("sql", "select count(1) TowerPositionConf"),
			zap.String("fields", cond+","+string(fields)),
			zap.String("error", err.Error()))
		return ret, err
	}

	return ret, err
}

