package model

import (
	"encoding/json"
	"github.com/jaryhe/gopkgs/logger"
	"github.com/jinzhu/gorm"
	"go.uber.org/zap"
	"time"
)

type TowerSensorConf struct {
	ID int64 `gorm:"column:ID;PRIMARY_KEY"`
	SN string `gorm:"column:SN"`
	ProjectID int64 `gorm:"column:ProjectID"`
	NearRangeAD float64 `gorm:"column:NearRangeAD"`
	NearRange float64 `gorm:"column:NearRange"`
	FarRangeAD float64 `gorm:"column:FarRangeAD"`
	FarRange float64 `gorm:"column:FarRange"`
	NearHighAD float64 `gorm:"column:NearHighAD"`
	NearHigh float64 `gorm:"column:NearHigh"`
	FarHighAD float64 `gorm:"column:FarHighAD"`
	FarHigh float64 `gorm:"column:FarHigh"`
	EmptyWeightAD int64 `gorm:"column:EmptyWeightAD"`
	EmptyWeight int64 `gorm:"column:EmptyWeight"`
	LoadWeightAD int64 `gorm:"column:LoadWeightAD"`
	LoadWeight int64 `gorm:"column:LoadWeight"`
	BackturnStartAD float64 `gorm:"column:BackturnStartAD"`
	BackturnStart float64 `gorm:"column:BackturnStart"`
	BackturnEndAD float64 `gorm:"column:BackturnEndAD"`
	BackturnEnd float64 `gorm:"column:BackturnEnd"`
	WindSpeedCoefficient float64 `gorm:"column:WindSpeedCoefficient"`
	TiltCoefficient float64 `gorm:"column:TiltCoefficient"`
	TowerNo int `gorm:"column:TowerNo"`
	CreatedAt time.Time `gorm:"column:CreatedAt"`
	UpdatedAt time.Time `gorm:"column:UpdatedAt"`
}

func (TowerSensorConf) TableName() string {
	return "TowerSensorConf"
}

func (p *TowerSensorConf) Insert(db *gorm.DB) error {
	err := db.Create(p).Error
	if err != nil {
		fields, _ := json.Marshal(p)
		logger.Error("mysql",
			zap.String("sql", "insert into TowerSensorConf"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return err
	}
	return err
}

func (p *TowerSensorConf) Find(db *gorm.DB, where map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}
	err = db.Table(p.TableName()).Where(cond, val...).First(p).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		logger.Error("mysql",
			zap.String("sql", "select from TowerSensorConf"),
			zap.String("fields", string(fields)),
			zap.String("error", err.Error()))
		return err
	}
	return err
}

func (p *TowerSensorConf) Update(db *gorm.DB, where map[string]interface{}, values map[string]interface{}) error {
	cond, val, err := whereBuild(where)
	if err != nil {
		return err
	}

	err = db.Table(p.TableName()).Where(cond, val...).Updates(values).Error
	if err != nil {
		fields, _ := json.Marshal(where)
		updateValues, _ := json.Marshal(values)
		logger.Error("mysql",
			zap.String("sql", "update TowerSensorConf"),
			zap.String("fields", string(fields)),
			zap.String("values", string(updateValues)),
			zap.String("error", err.Error()))
		return err
	}
	return err
}

func (p *TowerSensorConf) Count(db *gorm.DB, where map[string]interface{}, or map[string]interface{}) (int64, error) {

	cond, val, err := whereBuildAndOr(where, or)
	if err != nil {
		return 0, err
	}
	ret := int64(0)
	err = db.Table(p.TableName()).Where(cond, val...).Count(&ret).Error
	if err != nil {
		fields, _ := json.Marshal(val)
		logger.Error("mysql",
			zap.String("sql", "select count(1) TowerSensorConf"),
			zap.String("fields", cond+","+string(fields)),
			zap.String("error", err.Error()))
		return ret, err
	}

	return ret, err
}

